package encapsulation;

import java.util.List;

/**
 * TextWriter class, represents a writer that can write out metrics results
 * to a plain text file
 * @author Janina Voigt
 *
 */
public class TextWriter extends ResultsWriter {
	
	/**
	 * Constructor for TextWriter objects
	 * @param fileName the name of the file to write metrics data to
	 */
	public TextWriter(String fileName) {
		super(fileName);
	}
	
	/**
	 * Write out the header for a set of metrics values
	 * @param header the header to write out
	 */
	public void writeOutHeader(String header){
		writer.println(header);
		writer.println();
		writer.flush();
	}

	/**
	 * Write out a list of metrics results
	 * @param measurements the measurements to write out
	 */
	public void writeOutResults(List<Measurement> measurements){
		for(Measurement result : measurements){
			writer.println(result.getName() + ": " + result.getValue());
		}
		writer.println();
		writer.flush();
	}
}
