
package jst.symtab;

import java.util.*;
import java.lang.reflect.*;
import java.io.*;
import jst.symtab.visitor.ModelVisitor;
import jst.symtab.link.*;

/** *****************************************************************************
 * Definition of an array type.
 * Decorates another Type
 * Actual dimensions are ignored; there can be only one array type for any given type.
 */
public class ArrayType extends ReferenceType {


    /** The base type for the Array 
     * @supplierCardinality 1
     * @supplierRole baseType*/
    private TypeDecl baseType;

    /**
     * Every array has a length field.
     * @supplierCardinality 1 
     * @supplierRole length
     */
	private FieldDecl length;

    /**
     * Every array has a clone() method.
     * @supplierCardinality 1 
     * @supplierRole clone
     */
	private MethodDecl clone;

    /** Constructor to create a new array type */
    public ArrayType(TypeDecl theBaseType) {
    	super(theBaseType, theBaseType.getSimpleName() + "[]", null);
        baseType = theBaseType; // It already knows about this object.

        length = new FieldDecl(this, "length", null);
        length.setTypeRef(BuiltInTypeLink.getBoolean(new StringIdentifier("int", 0, 0)));
        length.setModifier(Modifier.PUBLIC);

		clone = new MethodDecl(this, "clone", null);

		ScopeLink link = new KnownDecl(this);
		link = new PackageLookup(link, new StringIdentifier("java", 0, 0));
		link = new PackageLookup(link, new StringIdentifier("lang", 0, 0));
		TypeLookup objectLookup = new ClassTypeLookup(link, new StringIdentifier("Object", 0, 0));

        clone.setTypeRef(objectLookup);

        clone.setModifier(Modifier.PUBLIC);				//UGLY!!! FIX LATER!!!
    }

	public void copy(UserType newOwnerUserType, Map<TypeParameter, ActualType> swaps) {
		assert false;
	}

    /** return the base type of the array */
    public TypeDecl getBaseType() {
    	return baseType;
    }

    public Scope getOwnerScope() {
    	return null;
    }
    
	public String getCanonicalName() {
    	return baseType.getCanonicalName() + "[]";
	}

	// Generate artificial name for anonymous decl in this scope.
    // Arrays do not contain anonymous members.
	public String generateName() {
    	throw new RuntimeException("Array should not contain anonymous declarations");
    }
	
	public ArrayType asArrayType() {
		return this;
	}

	public void addField(FieldDecl field) {
		// Do nothing.  This will be called only by constructor of length field.
		assert field == length;
    }

	public void addMethod(MethodDecl method) {
		// Do nothing.  This will be called only by constructor of clone() method.
		assert method == clone;
    }

	public boolean isSubtypeOf(ReferenceType parent) {
    	return parent == objct;
    }

	public boolean accessible(ReferenceType looker) {
       	return baseType.accessible(looker);
    }

	public boolean assignableFrom(TypeDecl rhs) {
        if (rhs instanceof NullType)
            return true;

		if (!(rhs instanceof ArrayType))
        	return false;

		ArrayType rhsArray = (ArrayType) rhs;
		return baseType.assignableFrom(rhsArray.getBaseType());
    }
	
	public boolean addType(UserType type) {
    	return false; // Cannot add inner type to an array.
    }

	public TypeDecl lookupType(boolean widen, String typeName, ReferenceType looker) {
    	return objct.lookupType(widen, typeName, looker);
    }

	public MethodDecl lookupMethod(boolean widen, String methodName, List params, ReferenceType looker) {
		if (methodName.equals("clone") && params.isEmpty())
        	return clone;

		return objct.lookupMethod(widen, methodName, params, looker);
    }

	public VariableDecl lookupVariable(boolean widen, String variableName, ReferenceType looker) {
		if (variableName.equals("length"))
        	return length;

		return objct.lookupVariable(widen, variableName, looker);
    }

    public void accept(ModelVisitor visitor) {
        visitor.visitArrayType(this);
    }
}

