
package jst.symtab;

import java.util.*;
import java.io.*;

import jst.symtab.visitor.ModelVisitor;

public class PrimitiveType extends TypeDecl {

	public static PrimitiveType BooleanType = new PrimitiveType("boolean", null);
	public static PrimitiveType VoidType    = new PrimitiveType("void", null);
	public static PrimitiveType DoubleType  = new PrimitiveType("double", null);
	public static PrimitiveType FloatType   = new PrimitiveType("float", DoubleType);
	public static PrimitiveType LongType    = new PrimitiveType("long", FloatType);
	public static PrimitiveType IntType     = new PrimitiveType("int", LongType);
	public static PrimitiveType CharType    = new PrimitiveType("char", IntType);
	public static PrimitiveType ShortType   = new PrimitiveType("short", IntType);
	public static PrimitiveType ByteType    = new PrimitiveType("byte", ShortType);

	public static PrimitiveType getPrimitiveType(String name) {
    	if (name.equals("byte"))    return ByteType;
    	if (name.equals("char"))    return CharType;
    	if (name.equals("double"))  return DoubleType;
    	if (name.equals("float"))   return FloatType;
    	if (name.equals("int"))     return IntType;
    	if (name.equals("long"))    return LongType;
    	if (name.equals("short"))   return ShortType;
    	if (name.equals("boolean")) return BooleanType;
    	if (name.equals("void"))    return VoidType;

		return null;
    }

	public static boolean isNumeric(TypeDecl type) {
    	return
        	type == DoubleType ||
        	type == FloatType  ||
            type == LongType   ||
            type == IntType    ||
            type == ShortType  ||
            type == ByteType;
    }

	private PrimitiveType promoteTo;

	private PrimitiveType(String name, PrimitiveType thePromoteTo) {
    	super(PackageDecl.getDefaultPackage(), name, null);
        promoteTo = thePromoteTo;
    }

	public void copy(UserType newOwnerUserType, Map<TypeParameter, ActualType> swaps) {
		assert false;
	}

 	public String getCanonicalName() {
       	return "TYP_" + simpleName;
    }

	public boolean assignableFrom(TypeDecl rhs) {
		if (!(rhs instanceof PrimitiveType))
        	return false;

		PrimitiveType prhs = (PrimitiveType) rhs;

		// Run up the chain of promotions
        for (PrimitiveType promoted = prhs; promoted != null; promoted = promoted.promoteTo)
			if (promoted == this)
				return true;

        return false;
    }

    public void accept(ModelVisitor visitor) {
        visitor.visitPrimitiveType(this);
    }
}
