
package jst.symtab;
import jst.parsetree.*;
import java.util.*;
import java.lang.reflect.Modifier;

public abstract class ReferenceType extends ActualType implements Scope {

    /**
     * Arrays inherit from Object.
     * Interfaces appear to inherit Object's methods.
     * This is ugly: how should we find Object?  Use lazy instantiation, so we have a symbol table
     * when we do the lookup.
     * @supplierCardinality 1 
     * @supplierRole object
     */
    public static UserType objct;

	protected ReferenceType(Decl theOwner, String theSimpleName, JstParseTreeBranch theSource) {
    	super(theOwner, theSimpleName, theSource);
    }

	protected ReferenceType(Decl newOwner, ReferenceType that, Map<TypeParameter, ActualType> swaps) {
    	super(newOwner, that, swaps);
    }

	public ReferenceType asReferenceType() {
		return this;
	}

	public Scope asScope() {
		return this;
	}

	public UserType asUserType() {
		return null;
	}
	
	public ArrayType asArrayType() {
		return null;
	}

	public abstract void addMethod(MethodDecl method);

	public abstract void addField(FieldDecl field);

	public abstract boolean isSubtypeOf(ReferenceType parent);

	public boolean isContainedBy(UserType container) {
    	for (ReferenceType content = this; content != null; content = content.getOwnerUserType()) {
        	if (content == container)
            	return true;
			if (Modifier.isStatic(content.getModifier())) // Can't see out of static inner class.
            	break;
        }
		return false;
    }
}
