package jst.symtab.link;

import jst.app.JSTMessage;
import jst.symtab.*;

// An array accass _may_ yield a scope.  
public class ArrayAccessLink extends ScopeLink implements TypeLink {

	protected TypeLink arrayTypeLink;

	public ArrayAccessLink(TypeLink theArrayTypeLink) {

		arrayTypeLink = theArrayTypeLink;

		assert arrayTypeLink != null;
	}

	public ArrayAccessLink(Decl newOwner, ArrayAccessLink that) {
		arrayTypeLink = that.arrayTypeLink.copyType(newOwner);
	}

	public ArrayAccessLink copy(Decl newOwner) {
		return new ArrayAccessLink(newOwner, this);
	}

	public ArrayAccessLink copyType(Decl newOwner) {
		return copy(newOwner);
	}

	public String getName() {
		return arrayTypeLink.getName();
	}

	public int getLine() {
		return arrayTypeLink.getLine();
	}

	public int getColumn() {
		return arrayTypeLink.getColumn();
	}

	public String getOwnerFileName() {
		return arrayTypeLink.getOwnerFileName();
	}

	public String getFullName() {
		return arrayTypeLink.getFullName() + "[-]";
	}

	public boolean isFirst() {
		return false;
	}

	public boolean isInstance() {
		return true;
	}

	public boolean resolve() {
		return arrayTypeLink.resolve();
	}

	public JSTMessage getMessage() {
		return arrayTypeLink.getMessage();
	}

	protected Scope getOriginScope() {
		// No way to work this out!
		// return arrayTypeLink.getOriginScope();
		return PackageDecl.getDefaultPackage(); // Just for now...
	}

	public Scope getScope() {
		return getType() == null? null : getType().asScope();
	}

	public Decl getDecl() {
		return getType();
	}

	public TypeDecl getType() {
		if (arrayTypeLink.getType() == null)
			return null;

		ArrayType array = arrayTypeLink.getType().asReferenceType().asArrayType();
		if (array == null)
			return null;

		return array.getBaseType();
	}
}
