package jst.symtab.link;

import jst.app.JSTMessage;
import jst.symtab.*;

public abstract class CompoundLookup extends Lookup {

	protected UserTypeLookup typeLookup;

	protected PackageLookup pkgLookup;
	
	public CompoundLookup(ScopeLink qualifier, Identifier name) {
		typeLookup = new UserTypeLookup(qualifier, name);
		pkgLookup = new PackageLookup(qualifier, name);
	}
	
	public CompoundLookup(Decl newOwner, CompoundLookup that) {
		typeLookup = that.typeLookup.copy(newOwner);
		pkgLookup = that.pkgLookup.copy(newOwner);
	}

	public String getName() {
		return pkgLookup.getName();
	}

	public int getLine() {
		return pkgLookup.getLine();
	}

	public int getColumn() {
		return pkgLookup.getColumn();
	}

	public String getOwnerFileName() {
		return pkgLookup.getOwnerFileName();
	}

	public String getFullName() {
		return pkgLookup.getFullName();
	}

	public boolean resolve() {
		return typeLookup.resolve() || pkgLookup.resolve();
	}

	public Decl getDecl() {
		if (getUserType() != null)
			return getUserType();

		return getPackageDecl();
	}

	public PackageDecl getPackageDecl() {
		return pkgLookup.getPackageDecl();
	}

	public UserType getUserType() {
		return typeLookup.getUserType();
	}

	public Scope getScope() {
		if (getUserType() != null)
			return getUserType();

		return getPackageDecl();
	}

	public boolean isFirst() {
		return pkgLookup.isFirst();
	}

	public boolean isInstance() {
		return false;
	}

	protected Scope getOriginScope() {
		return pkgLookup.getOriginScope();
	}

	public JSTMessage getMessage() {
		// return new JSTMessage(getOwnerFileName(), getLine(), getColumn(), getError(), getName());
		return pkgLookup.getMessage();
	}

	protected JSTMessage getError() {
		return JSTMessage.PACKAGE_OR_TYPE_NOT_FOUND;
	}

	public String toString() {
		return typeLookup.toString();
	}
}
