package jst.symtab.link;

import java.util.*;
import jst.app.JSTMessage;
import jst.symtab.*;

public abstract class OperationLookup extends TypedLookup {
	
	protected List<ReferenceTypeLink> typeArgLinks;

	protected List<TypeLink> args;
	
	public OperationLookup(ScopeLink theQualifier, Identifier name) {
		super(theQualifier, name);

		typeArgLinks = new ArrayList<ReferenceTypeLink>();
		args = new ArrayList<TypeLink>();
	}

	public OperationLookup(Decl newOwner, OperationLookup that) {
		super(newOwner, that);

		typeArgLinks = new ArrayList<ReferenceTypeLink>();
		args = new ArrayList<TypeLink>();

		for (ReferenceTypeLink thatTypeArg : that.typeArgLinks)
			typeArgLinks.add(thatTypeArg.copyType(newOwner));

		for (TypeLink thatArg : that.args)
			args.add(thatArg.copyType(newOwner));
	}

	public String getName() {
		String argNames = "";

		Iterator<TypeDecl> argIter = getArgTypes().iterator();
		while (argIter.hasNext()) {
			TypeDecl argType = argIter.next();

			argNames += argType == null? "UNKNOWN" : argType.getSimpleName();
			if (argIter.hasNext())
				argNames += ", ";
		}

		return super.getName() + "(" + argNames + ")";
	}

	public void addTypeArgument(ReferenceTypeLink typeArg) {
		typeArgLinks.add(typeArg);
	}

	protected boolean resolveTypeArgs() {

		for (ReferenceTypeLink typeArg: typeArgLinks)
			if (!typeArg.resolve())
				return false;

		return true;
	}

	protected List<TypeDecl> getTypeArgs() {
		List<TypeDecl> typeArgs = new ArrayList<TypeDecl>();

		for (ReferenceTypeLink typeArg: typeArgLinks)
			typeArgs.add(typeArg.getType());

		return typeArgs;
	}

	public void addArgument(TypeLink arg) {
		assert arg != null;

		args.add(arg);
	}
	
	protected boolean resolveArgs() {
		for (Link arg: args)
			if (!arg.resolve())
				return false;

		return true;
	}

	protected List<TypeDecl> getArgTypes() {
		List<TypeDecl> argTypes = new ArrayList<TypeDecl>();

		for (TypeLink arg: args)
			argTypes.add(arg.getType());

		return argTypes;
	}
	
	public List<TypeLink> getArgLinks() {
		return Collections.unmodifiableList(args);
	}

	public JSTMessage getMessage() {

		for (Link arg: args) {
			JSTMessage argMessage = arg.getMessage();
			if (argMessage != null)
				return argMessage;
		}

		return super.getMessage();
	}

	public Decl getDecl() {
		return getOperationDecl();
	}

	public abstract OperationDecl getOperationDecl();

	public Scope getScope() {
		if (getType() == null)
			return null;

		return getType().asReferenceType();
	}

	public TypeDecl getType() {
		if (getOperationDecl() == null)
			return null;

		return getOperationDecl().getType();
	}
}
