package jst.symtab.link;

import java.lang.reflect.Modifier;

import jst.app.JSTMessage;
import jst.symtab.*;

public class PackageLookup extends IdLookup {

	protected PackageDecl pkg;

	public PackageLookup(ScopeLink theQualifier, Identifier name) {
		super(theQualifier, name);
	}

	public PackageLookup(Decl newOwner, PackageLookup that) {
		super(newOwner, that);
	}

	public PackageLookup copy(Decl newOwner) {
		return new PackageLookup(newOwner, this);
	}

	// Need special resolve() because Java reflection API sucks.
	// Reflection will find "java.lang", but not "java".
	// Need whole name.
	public boolean resolve() {
		if (!qualifier.resolve()) {
			// Special whole-name reflection.
		   	// return false;
			return reflect();
		}
	
		if (!lookup())
			return reflect();

		return true;
	}

	protected boolean lookup() {
		PackageDecl ownerPkg;

		if (widen())
			ownerPkg = PackageDecl.getDefaultPackage();
		else {
			if(qualifier.getScope() instanceof PackageDecl){
				ownerPkg = (PackageDecl) qualifier.getScope(); 
			}
			else {
				return false;
			}
		}

		pkg = ownerPkg.lookupPackage(getName());

		return pkg != null;
	}

	// Side-effect: sets pkg.
	protected boolean reflect() {

		Package javaPkg = Package.getPackage(getFullName());
		if (javaPkg == null)
			return false;

		pkg = PackageDecl.getDefaultPackage();

		// Make ALL relevant packages.
		String[] pkgNames = getFullName().split("\\.");
		for (String pkgName: pkgNames) {
			PackageDecl kidPkg = pkg.lookupPackage(pkgName);

			if (kidPkg == null)
				kidPkg = pkg.addPackage(pkgName, null);

			pkg = kidPkg;
		}

		return true;
	}

	protected JSTMessage getError() {
		return JSTMessage.PACKAGE_NOT_FOUND;
	}

	public Decl getDecl() {
		return getPackageDecl();
	}

	public PackageDecl getPackageDecl() {
		return pkg;
	}

	public Scope getScope() {
		return pkg;
	}

	public boolean isInstance() {
		return false;
	}
}
