package jst.symtab.link;

import java.util.*;
import java.lang.reflect.Modifier;

import jst.app.JSTMessage;
import jst.symtab.*;

public class TypeLookup extends IdLookup implements ReferenceTypeLink {
	
	protected List<ActualTypeLink> typeArgs;
	
	protected ReferenceType type; // UserType or TypeParameter

	protected boolean isInstance; // Needed for cast expressions.

	public TypeLookup(ScopeLink theQualifier, Identifier name) {
		super(theQualifier, name);

		typeArgs = new ArrayList<ActualTypeLink>();
		isInstance = false;
	}

	public TypeLookup(Decl newOwner, TypeLookup that) {
		super(newOwner, that);

		typeArgs = new ArrayList<ActualTypeLink>();
		isInstance = that.isInstance;

		for (ActualTypeLink thatArg : that.typeArgs)
			typeArgs.add(thatArg.copyType(newOwner));
	}

	public TypeLookup copy(Decl newOwner) {
		return new TypeLookup(newOwner, this);
	}

	public TypeLookup copyType(Decl newOwner) {
		return copy(newOwner);
	}

	public void addTypeArg(ActualTypeLink arg) {
		typeArgs.add(arg);
	}

	protected boolean lookup() {
		if (!resolveArgTypes())
			return false;
		
		if(qualifier.getScope() == null) {
			return false;
		}

		TypeDecl typeDecl = qualifier.getScope().lookupType(widen(), getName(), getLooker());

		// Fix this
		type = (ReferenceType) typeDecl;

		if (type == null)
			return false;

		if (!typeArgs.isEmpty()) {

			UserType userType = type.asUserType();

			if (userType == null)
				return false;

			// Quick hack
			if (userType instanceof GenericClassType)
				type = ((GenericClassType) userType).parameterize(getArgTypes());
			else
				type = ((GenericInterfaceType) userType).parameterize(getArgTypes());
		}

		return checkType();
	}

	/*
	protected boolean reflect() {

		try {
    		Class clazz = Class.forName(getFullName());
        }
        catch (Exception e) {
	    	return false;
		}

		return true;
	}
	*/

	protected boolean resolveArgTypes() {

		for (ActualTypeLink arg: typeArgs)
			if (!arg.resolve())
				return false;

		return true;
	}

	public List<ActualType> getArgTypes() {
		List<ActualType> resolvedArgs = new ArrayList<ActualType>(typeArgs.size());

		for (ActualTypeLink arg: typeArgs)
			resolvedArgs.add(arg.getActualType());

		return resolvedArgs;
	}

	protected boolean checkType() {
		return true;
	}

	public JSTMessage getMessage() {

		if (qualifier.getDecl() == null)
			return qualifier.getMessage();

		for (ActualTypeLink arg: typeArgs)
			if (arg.getType() == null)
				return arg.getMessage();

		return super.getMessage();
	}

	protected JSTMessage getError() {
		if (type == null)
			return JSTMessage.TYPE_NOT_FOUND;

		return getTypeError();
	}

	protected JSTMessage getTypeError() {
		return null;
	}

	public Decl getDecl() {
		return getType();
	}

	public TypeDecl getType() {
		return getActualType();
	}

	public ActualType getActualType() {
		return getReferenceType();
	}

	public ReferenceType getReferenceType() {
		if (!checkType())
			return null;

		return type;
	}

	public Scope getScope() {
		return getReferenceType();
	}

	public void setIsInstance() {
		isInstance = true; // This type is being used as a cast of an instance.
	}

	public boolean isInstance() {
		return isInstance;
	}
}
