package jst.symtab.visitor;
import java.io.PrintWriter;
import java.lang.reflect.Modifier;
import java.util.*;
import jst.parsetree.*;
import jst.symtab.*;
import jst.symtab.link.*;

public class XMLWriter extends CompositionVisitor {

    protected PrintWriter out;

    protected String indent;

    public XMLWriter(PrintWriter theOut) {
        out = theOut;
        indent = "";
    }

    public void visitPackageDecl(PackageDecl packageDecl) {
        out.print(indent + "<package id='" + packageDecl.getCanonicalName() + "' ");
	    out.print("name='" + packageDecl.getSimpleName() + "' sources='");

        Iterator srcIter = packageDecl.getAllSources().iterator();
        while (srcIter.hasNext()) {
	        JstParseTreeBranch src = (JstParseTreeBranch) srcIter.next();
            out.print(src.getID());
            if (srcIter.hasNext())
                out.print(" ");
        }

        out.println("'>");

        super.visitPackageDecl(packageDecl);

        out.println(indent + "</package>");
    }

    public void visitSourceFile(SourceFile sourceFile) {
	    out.println(indent + "<sourcefile name='" + sourceFile.getCanonicalName() + "'>");

        super.visitSourceFile(sourceFile);

        sourceFile.getParseTree().save(out, indent + " ");
	    out.println(indent + "</sourcefile>");
    }

    public void visitArrayType(ArrayType arrayType) {
        out.println(
            indent +
            "<array id='" + arrayType.getCanonicalName() + "' " +
            "name='" + arrayType.getSimpleName() + "' " +
            "baseType='" + arrayType.getBaseType().getCanonicalName() + "'/>"
        );
    }

    public void visitInterfaceType(InterfaceType interfaceType) {
        if (interfaceType.getSource() == null)
            return;

        out.print(indent + "<interface id='" + interfaceType.getCanonicalName() +
	        "' name='" + interfaceType.getSimpleName() +
         "' source='" + (interfaceType.getSource() == null? "" : interfaceType.getSource().getID()) + "'");
	    out.print(" superinterfaces='");

        Iterator ifaceIter = interfaceType.getSuperInterfaces().iterator();
	    while (ifaceIter.hasNext()) {
	        InterfaceType iface = (InterfaceType) ifaceIter.next();
            out.print(iface.getCanonicalName());
            if (ifaceIter.hasNext())
             out.print(",");
        }

        out.println("' modifier='" + Modifier.toString(interfaceType.getModifier()) + "'>");

        super.visitInterfaceType(interfaceType);

        out.println(indent + "</interface>");
    }

    public void visitClassType(ClassType classType) {
        if (classType.getSource() == null)
            return;

        out.print(indent + "<class id='" + classType.getCanonicalName() + "' ");
	    out.print("name='" + classType.getSimpleName() + "' ");
        out.print("source='" + (classType.getSource() == null? "" : classType.getSource().getID()) + "' ");
	    out.print("superclass='");

        if (classType.getSuperclass() != null)
            out.print(classType.getSuperclass().getCanonicalName());

        out.print("' implements='");

        Iterator ifaceIter = classType.getInterfaces().iterator();
	    while (ifaceIter.hasNext()) {
	        InterfaceType iface = (InterfaceType) ifaceIter.next();
            out.print(iface.getCanonicalName());
            if (ifaceIter.hasNext())
             out.print(",");
        }

        out.println("' modifier='" + Modifier.toString(classType.getModifier()) + "'>");

        super.visitClassType(classType);

        out.println(indent + "</class>");
    }

    public void visitFieldDecl(FieldDecl fieldDecl) {
        out.println(indent + "<field id='" + fieldDecl.getCanonicalName() + "' " +
            "name='" + fieldDecl.getSimpleName() + "' " +
            "source='" + (fieldDecl.getSource() == null? "" : fieldDecl.getSource().getID()) + "' " +
         "type='" + (fieldDecl.getType() == null? "UNKNOWN" : fieldDecl.getType().getCanonicalName()) + "' modifier='" + Modifier.toString(fieldDecl.getModifier()) + "'/>");
    }

    public void visitLocalVariableDecl(LocalVariableDecl localVariableDecl) {
        out.println(indent + "<variable id='" + localVariableDecl.getCanonicalName() + "' " +
            "name='" + localVariableDecl.getSimpleName() + "' " +
            "source='" + (localVariableDecl.getSource() == null? "" : localVariableDecl.getSource().getID()) + "' " +
         "type='" + (localVariableDecl.getType() == null? "UNKNOWN" : localVariableDecl.getType().getCanonicalName()) + "'/>");
    }

    public void visitParameterDecl(ParameterDecl parameterDecl) {
        out.println(indent + "<parameter id='" + parameterDecl.getCanonicalName() + "' " +
            "name='" + parameterDecl.getSimpleName() + "' " +
            "source='" + (parameterDecl.getSource() == null? "" : parameterDecl.getSource().getID()) + "' " +
         "type='" + (parameterDecl.getType() == null? "UNKNOWN" : parameterDecl.getType().getCanonicalName()) + "' modifier='" + Modifier.toString(parameterDecl.getModifier()) + "'/>");
    }

    public void visitMethodDecl(MethodDecl methodDecl) {
        out.println(indent +
         "<method id='" + methodDecl.getCanonicalName() + "' " +
            "name='" + methodDecl.getSimpleName() + "()' " +
            "source='" + (methodDecl.getSource() == null? "" : methodDecl.getSource().getID()) + "' " +
         "type='" + (methodDecl.getType() == null? "UNKNOWN" : methodDecl.getType().getCanonicalName()) + "' modifier='" + Modifier.toString(methodDecl.getModifier()) + "'>");

        super.visitMethodDecl(methodDecl);

        out.println(indent + "</method>");
    }

    public void visitConstructorDecl(ConstructorDecl constructorDecl) {
        out.println(indent +
            "<constructor id='" + constructorDecl.getCanonicalName() + "' " +
            "name='" + constructorDecl.getSimpleName() + "()' " +
            "source='" + (constructorDecl.getSource() == null? "" : constructorDecl.getSource().getID()) + "' modifier='" + Modifier.toString(constructorDecl.getModifier()) + "'>");

        super.visitConstructorDecl(constructorDecl);

        out.println(indent + "</constructor>");
    }

    public void visitBlock(Block block) {
        out.println(indent + "<block id='" + block.getCanonicalName() + "'>");

        super.visitBlock(block);

		for (Link link: block.getReferences()) {
            out.print(indent + " <reference to='" + link.getName() + "' from='");

			out.print(" Should be source code ID here (" + link.getLine() + ", " + link.getColumn() + ")' ");

			String declName = link.getDecl() == null? "UNKNOWN" : link.getDecl().getCanonicalName();
			out.print("resolved='" + declName);

            out.println("'/>");
        }

        out.println(indent + "</block>");
    }

    public void visitCatchBlock(CatchBlock catchBlock) {
        out.println(indent + "<catch id='" + catchBlock.getCanonicalName() + "'>");

        super.visitCatchBlock(catchBlock);

        out.println(indent + "</catch>");
    }

    public void visitFieldInitializer(FieldInitializer fieldInitializer) {
        out.println(indent + "<initializer id='" + fieldInitializer.getCanonicalName() + "'>");

        super.visitFieldInitializer(fieldInitializer);

        out.println(indent + "</initializer>");
    }

}
