package parser.grammar;

import java.util.*;

/** Represents a nonterminal in a grammar (i.e. a lhs of a production). */
public class Nonterminal extends Symbol {

	protected boolean startSymbol;

    /** Productions that use this Symbol on the lhs.
     * @associates parser.grammar.Production
     * @supplierRole productions
     * @supplierCardinality 1..*
     * @link aggregation
     */
    protected List productions; // Productions.

    /**
     * Construct a Nonterminal with the given name.
     *
     * @param theName describes this Nonterminal.
     */
    public Nonterminal(String theName) {
	    this(theName, 0);
	}

    public Nonterminal(String theName, int theID) {
	    super(theName, theID);
        productions = new ArrayList();
	}

    /**
     * Is this a Terminal?
     *
     * @return false, always.
     */
	public boolean isTerminal() {
		return false;
	}

    public Nonterminal asNonterminal() {
        return this;
    }

	public void setStartSymbol(boolean truth) {
		startSymbol = truth;
	}

	public boolean isStartSymbol() {
		return startSymbol;
	}

    /**
     * Tell this Nonterminal it is used on a lhs.
     * This is called by a Production only.
     *
     * @param production A production that uses this Nonterminal on its left.
     */
    public void addProduction(Production production) {
        productions.add(production);
    }

    /** Getter. */
    public List getProductions() {
        return productions;
    }

    public boolean isShift() {
        return false;
    }

    public Terminal asShift() {
        return null;
    }

    public Nonterminal asGoto() {
        return this;
    }
}
