package parser.parsetree;

import parser.parsetree.visitor.*;
import parser.grammar.Symbol;
import java.util.*;

/** A node in a syntax tree.  May be a parse tree or AST. */
public interface SyntaxTree {

    /**
     * Accept a visitor.  GoF Visitor pattern variant.
     *
     * @param visitor the visitor.
     */
    public void accept(SyntaxTreeVisitor visitor);

    /** Getter for the type of symbol contained by this node. */
	public Symbol getSymbolType();

    /**
     * Getter for line on which value starts.
     * <p>
     * Empty trees return 0, because they don't know where they are.
     */
	public int getLine();

    /**
     * Getter for column on which value starts.
     * <p>
     * Empty trees return 0, because they don't know where they are.
     */
	public int getColumn();

    /** Get the string value of the tree.  I.e. the contents of the leaves. */
    public String getValue();
    
    public void setParent(SyntaxTreeBranch theParent);
    
    public SyntaxTreeBranch getParent();

    public SyntaxTreeBranch asBranch();

    public SyntaxTreeLeaf asLeaf();
}
