package parser.parsetree;

import parser.grammar.*;
import java.util.*;

/**
* SyntaxTreeBranch
* <p>
* There are two main ways to implement parse trees: (1) use a generic node for all
* productions, or (2) use production-specific branches.  The second requires code generation
* from the grammar. E.g. a code generator can subclass ParseTreeBranch to make specific
* parse tree branches, so for decl ::= type variable ;
*     class DeclNode extends ParseTreeBranch {
*     }
*     class DeclNode1 extends DeclNode {
*         TypeNode type;
*         VariableNode variable;
*         ParseTreeLeaf semi;
*     }
*/
public abstract class SyntaxTreeArrayBranch extends AbstractSyntaxTreeBranch {

	protected SyntaxTreeBranch parent;
	
    /**
     * The subtrees contained in this branch.
     * 
     * @supplierCardinality 0..*
     * @supplierRole kids
     * @link aggregation
	 */
    protected SyntaxTree[] kids;

	protected SyntaxTreeArrayBranch(SyntaxTree[] theKids) {
        assert theKids != null;

        kids = theKids;

		for (SyntaxTree kid : kids)
			kid.setParent(this);
    }
 
	public void setParent(SyntaxTreeBranch theParent) {
		parent = theParent;
	}
	
	public SyntaxTreeBranch getParent() {
		return parent;
	}

    /**
     * Getter for the kid nodes.
     *
     * @return array containing SyntaxTreeNodes.
     */
    public SyntaxTree[] getKids() {
        return kids;
    }
    
    public void setKids(SyntaxTree[] theKids){
    	kids = theKids;
    }
}
