package parser.parsetree.treeFactory;

import parser.grammar.*;
import parser.parsetree.*;

import java.util.*;

/**
* Syntax tree branch factory that makes GenericParseTreeBranches.
* These work for any grammar, and exactly reflect the grammar (unlike ASTs).
*/
public class GenericParseTreeFactory implements SyntaxTreeFactory {

    /**
     * Create a vanilla parse tree branch node.
     *
     * @param type the Type of production recognised.
     * @param kids the child nodes of this branch.
     * @return a new branch node.
     */
    public SyntaxTreeBranch makeBranch(Production type,List kids) {
        return new ParseTreeBranch(type, kids);
    }

    /**
     * Replace an existing node with an ambiguous one.  Checks first whether the alternative is identical.
     *
     * @param existing A parse tree node that may already be in use in a tree.
     * @param alternative An ambiguous alternative that has the same lhs.
     */
	public void makeAmbiguity(SyntaxTreeBranch existing, SyntaxTreeBranch alternative) {
        existing.ambiguity(alternative);
    }
}
