package parser.parsetree.visitor;

import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.parsers.ParserConfigurationException;

import parser.parsetree.*;
import parser.grammar.*;

public class XMLVisitor extends SyntaxTreeVisitor {
    protected Element targetElement;
    protected Document doc;

	protected Terminal whitespaceTerminal;

    public XMLVisitor(Terminal theWhitespaceTerminal) {
        try {
            doc = javax.xml.parsers.DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        }
        catch (ParserConfigurationException e) {
            throw new RuntimeException(e);
        }

        targetElement = doc.createElement("parsetree");
        doc.appendChild(targetElement);

		whitespaceTerminal = theWhitespaceTerminal;
    }

    public Document getXMLDocument() {
        return doc;
    }

    public void visitLeaf(SyntaxTreeLeaf leaf) {
        String label = leaf.getSymbolType().toString();

        /*
        Element newElem = doc.createElement(label);
        targetElement.appendChild(newElem);

        newElem.appendChild(doc.createCDATASection(leaf.getValue()));
        */

        Element newElem = doc.createElement("token");
        targetElement.appendChild(newElem);

        newElem.setAttribute("id", label);
        newElem.setAttribute("line", String.valueOf(leaf.getLine()));
        newElem.setAttribute("column", String.valueOf(leaf.getColumn()));
        newElem.appendChild(doc.createCDATASection(leaf.getValue()));

		// Also put out any trailing whitespace -- it isn't in the tree.
		SyntaxTreeLeaf trailing = leaf.getNext();
		if (trailing != null && trailing.getTerminalType() == whitespaceTerminal)
			trailing.accept(this);
    }

    public void visitBranch(SyntaxTreeBranch branch) {
        Element oldTarget = targetElement;

        if (branch.isAmbiguous())
            targetElement = doc.createElement("AMBIGUOUS");
        else if (branch.isList())
           targetElement = doc.createElement("LIST");
        else
           targetElement = doc.createElement(branch.getSymbolType().toString());

        oldTarget.appendChild(targetElement);

        super.visitBranch(branch); // Do kids.

        targetElement = oldTarget;
    }
}
