package parser.pda;

import parser.parsetree.*;
import parser.grammar.*;
import parser.pda.stateMachine.*;
import parser.pda.tokenStream.*;

import java.util.*;
import util.MultiMap;

public abstract class GSSNode extends StateStackNode {

    /**
     * @supplierRole parser
     * @supplierCardinality 1 
     */
    protected NondeterministicParser parser;

    public GSSNode(NondeterministicParser theParser, State theState) {
        super(theState);
        parser = theParser;
    }

    public void shift(Token input) {
        State dst = state.getTransition(input.getTerminalType());

        if (dst != null)
            parser.shift(this, input, dst);
    }

    public GSSNode makeShiftNode(State dst, SyntaxTree tree) {
        return new ShiftNode(parser, this, dst, tree);
    }

    public void goTo(State dst, Production prod, List kids, Looker input) {
        parser.goTo(this, dst, prod, kids, input);
    }

    public GSSNode makeGotoNode(State dst, SyntaxTree tree) {
        return new GotoNode(parser, this, dst, tree);
    }

    public GSSNode checkCycle(State dst) {
        return null;
    }

    public void addPrevious(GSSNode prev, SyntaxTree tree) {
        assert false;
    }

    public abstract SyntaxTree getTree(GSSNode prev);
}
