package parser.pda;

import parser.parsetree.*;
import parser.grammar.*;
import parser.pda.stateMachine.*;
import parser.pda.tokenStream.*;
import java.util.*;
import util.MultiMap;

public class LinearStackNode extends StateStackNode {

    /**
     * @supplierRole parser
     * @supplierCardinality 1 
     */
    protected DeterministicParser parser;

    /**
     * @supplierRole previous
     * @supplierCardinality 0..1 
     */
    protected LinearStackNode previous;

    protected SyntaxTree tree;

    // Make root node.
    public LinearStackNode(DeterministicParser theParser, State theState) {
        this(theParser, null, theState, null);
    }

    public LinearStackNode(DeterministicParser theParser, LinearStackNode prev, State theState, SyntaxTree theTree) {
        super(theState);
        parser = theParser;
        previous = prev;
        tree = theTree;
    }

    /*
    public void act(Looker looker) {
        if (state.accepts()) {
            SyntaxTreeBranch root = (SyntaxTreeBranch) tree;
            SyntaxTreeBranch firstKid = (SyntaxTreeBranch) root.getKids()[0];
            stack.setResult(firstKid);
        }
        else
            super.act(looker);
    }
    */

    public void shift(Token input) {
        State dst = state.getTransition(input.getTerminalType());
        parser.shift(input, dst);
    }

    public LinearStackNode makeShiftNode(State dst, SyntaxTree tree) {
        return new LinearStackNode(parser, this, dst, tree);
    }

    protected void reducePrevious(int n, Production prod, List kids, Looker input) {
        kids.add(0, tree);
        previous.reduce(n, prod, kids, input);
        kids.remove(0);
	}

    protected void goTo(State dst, Production prod, List kids, Looker input) {
        parser.goTo(this, dst, prod, kids, input);
    }

    public LinearStackNode makeGotoNode(State dst, SyntaxTree tree) {
        return new LinearStackNode(parser, this, dst, tree);
    }

    public SyntaxTreeBranch getResult() {
        if (state.getSymbol().isTerminal())
            return previous.getResult(); // Back up to first Nonterminal

        return tree.asBranch();
    }
}
