package parser.pda.stateMachine;

import parser.parsetree.*;
import parser.pda.tokenStream.*;
import parser.grammar.*;

import java.util.*;
import parser.pda.*;
import parser.pda.PdaActor;

public class DeterministicKLookahead extends KLookahead {

    protected ShiftOrReduce action; // May be null, in which case check kids.

    /**
     * @link aggregation
     * @associates parser.pda.stateMachine.DeterministicKLookahead
     * @supplierRole next
     * @supplierCardinality 0..*
     */
    protected Map next; // Terminal ==> DeterministicKLookahead

    public DeterministicKLookahead() {
        next = new HashMap();
    }

    // For builder only.
    public void addAction(ShiftOrReduce theAction) {
        assert theAction != null && action == null;

        action = theAction;
    }

    public void act(Looker input, PdaActor pda) {
        if (next.isEmpty()) {
			act(action, input, pda);
            return;
        }

		Terminal terminal = input.look();
		KLookahead kid = (KLookahead) next.get(terminal);
		if (kid != null)
			kid.act(input, pda);
    }

    public KLookahead grow(Terminal symbol) {
        KLookahead result = (DeterministicKLookahead) next.get(symbol);

        if (result == null) {
            result = new DeterministicKLookahead();
            next.put(symbol, result);
        }

        return result;
    }

    protected Map getNext() {
        return next;
    }

    public String toString() {
        String result = "";

        if (action != null)
            result += action.toString();

        return result + super.toString();
    }
}
